@@ -18,12 +18,19 @@
 # perhaps show them in log -v and allow them as options to the commit command.
 
 
-import bzrlib.errors as errors
-from bzrlib.graph import node_distances, select_farthest, all_descendants, Graph
+from bzrlib import (
+    errors,
+    symbol_versioning,
+    )
+from bzrlib.deprecated_graph import (
+    all_descendants,
+    Graph,
+    node_distances,
+    select_farthest,
+    )
 from bzrlib.osutils import contains_whitespace
 from bzrlib.progress import DummyProgress
 from bzrlib.symbol_versioning import (deprecated_function,
-        zero_eight,
         )
 
 NULL_REVISION="null:"
@@ -107,9 +114,18 @@ class Revision(object):
     def get_summary(self):
         """Get the first line of the log message for this revision.
         """
-        return self.message.split('\n', 1)[0]
+        return self.message.lstrip().split('\n', 1)[0]
+
+    def get_apparent_author(self):
+        """Return the apparent author of this revision.
+
+        If the revision properties contain the author name,
+        return it. Otherwise return the committer name.
+        """
+        return self.properties.get('author', self.committer)
 
 
+@deprecated_function(symbol_versioning.one_zero)
 def is_ancestor(revision_id, candidate_id, branch):
     """Return true if candidate_id is an ancestor of revision_id.
 
@@ -118,8 +134,11 @@ def is_ancestor(revision_id, candidate_id, branch):
     
     revisions_source is an object supporting a get_revision operation that
     behaves like Branch's.
+
+    This function is deprecated, it is better for callers to directly use
+    Graph.is_ancestor() (just watch out that the parameter order is switched)
     """
-    return (candidate_id in branch.repository.get_ancestry(revision_id))
+    return branch.repository.get_graph().is_ancestor(candidate_id, revision_id)
 
 
 def iter_ancestors(revision_id, revision_source, only_present=False):
@@ -214,6 +233,7 @@ def _revision_graph(revision, revision_source):
     return root, ancestors, descendants
 
 
+@deprecated_function(symbol_versioning.one_three)
 def combined_graph(revision_a, revision_b, revision_source):
     """Produce a combined ancestry graph.
     Return graph root, ancestors map, descendants map, set of common nodes"""
@@ -237,6 +257,7 @@ def combined_graph(revision_a, revision_b, revision_source):
     return root, ancestors, descendants, common
 
 
+@deprecated_function(symbol_versioning.one_three)
 def common_ancestor(revision_a, revision_b, revision_source, 
                     pb=DummyProgress()):
     if None in (revision_a, revision_b):
@@ -251,11 +272,19 @@ def common_ancestor(revision_a, revision_b, revision_source,
             pb.update('Picking ancestor', 1, 3)
             graph = revision_source.get_revision_graph_with_ghosts(
                 [revision_a, revision_b])
+            # Shortcut the case where one of the tips is already included in
+            # the other graphs ancestry.
+            ancestry_a = graph.get_ancestry(revision_a, topo_sorted=False)
+            if revision_b in ancestry_a:
+                return revision_b
+            ancestry_b = graph.get_ancestry(revision_b, topo_sorted=False)
+            if revision_a in ancestry_b:
+                return revision_a
             # convert to a NULL_REVISION based graph.
             ancestors = graph.get_ancestors()
             descendants = graph.get_descendants()
-            common = set(graph.get_ancestry(revision_a)).intersection(
-                     set(graph.get_ancestry(revision_b)))
+            common = set(ancestry_a)
+            common.intersection_update(ancestry_b)
             descendants[NULL_REVISION] = {}
             ancestors[NULL_REVISION] = []
             for root in graph.roots:
@@ -293,6 +322,8 @@ def common_ancestor(revision_a, revision_b, revision_source,
 
 class MultipleRevisionSources(object):
     """Proxy that looks in multiple branches for revisions."""
+
+    @symbol_versioning.deprecated_method(symbol_versioning.one_three)
     def __init__(self, *args):
         object.__init__(self)
         assert len(args) != 0
@@ -411,46 +442,34 @@ class MultipleRevisionSources(object):
             source.unlock()
 
 
-@deprecated_function(zero_eight)
-def get_intervening_revisions(ancestor_id, rev_id, rev_source,
-                              revision_history=None):
-    """Find the longest line of descent from maybe_ancestor to revision.
-    Revision history is followed where possible.
+def is_reserved_id(revision_id):
+    """Determine whether a revision id is reserved
 
-    If ancestor_id == rev_id, list will be empty.
-    Otherwise, rev_id will be the last entry.  ancestor_id will never appear.
-    If ancestor_id is not an ancestor, NotAncestor will be thrown
+    :return: True if the revision is is reserved, False otherwise
     """
-    root, ancestors, descendants = revision_graph(rev_id, rev_source)
-    if len(descendants) == 0:
-        raise errors.NoSuchRevision(rev_source, rev_id)
-    if ancestor_id not in descendants:
-        rev_source.get_revision(ancestor_id)
-        raise errors.NotAncestor(rev_id, ancestor_id)
-    root_descendants = all_descendants(descendants, ancestor_id)
-    root_descendants.add(ancestor_id)
-    if rev_id not in root_descendants:
-        raise errors.NotAncestor(rev_id, ancestor_id)
-    distances = node_distances(descendants, ancestors, ancestor_id,
-                               root_descendants=root_descendants)
+    return isinstance(revision_id, basestring) and revision_id.endswith(':')
 
-    def best_ancestor(rev_id):
-        best = None
-        for anc_id in ancestors[rev_id]:
-            try:
-                distance = distances[anc_id]
-            except KeyError:
-                continue
-            if revision_history is not None and anc_id in revision_history:
-                return anc_id
-            elif best is None or distance > best[1]:
-                best = (anc_id, distance)
-        return best[0]
 
-    next = rev_id
-    path = []
-    while next != ancestor_id:
-        path.append(next)
-        next = best_ancestor(next)
-    path.reverse()
-    return path
+def check_not_reserved_id(revision_id):
+    """Raise ReservedId if the supplied revision_id is reserved"""
+    if is_reserved_id(revision_id):
+        raise errors.ReservedId(revision_id)
+
+
+def ensure_null(revision_id):
+    """Ensure only NULL_REVISION is used to represent the null revision"""
+    if revision_id is None:
+        symbol_versioning.warn('NULL_REVISION should be used for the null'
+            ' revision instead of None, as of bzr 0.91.',
+            DeprecationWarning, stacklevel=2)
+        return NULL_REVISION
+    else:
+        return revision_id
+
+
+def is_null(revision_id):
+    if revision_id is None:
+        symbol_versioning.warn('NULL_REVISION should be used for the null'
+            ' revision instead of None, as of bzr 0.90.',
+            DeprecationWarning, stacklevel=2)
+    return revision_id in (None, NULL_REVISION)
