@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ import re
 
 from bzrlib import (
     errors,
+    osutils,
     revision,
     symbol_versioning,
     trace,
@@ -93,6 +94,18 @@ class RevisionInfo(object):
         return '<bzrlib.revisionspec.RevisionInfo object %s, %s for %r>' % (
             self.revno, self.rev_id, self.branch)
 
+    @staticmethod
+    def from_revision_id(branch, revision_id, revs):
+        """Construct a RevisionInfo given just the id.
+
+        Use this if you don't know or care what the revno is.
+        """
+        try:
+            revno = revs.index(revision_id) + 1
+        except ValueError:
+            revno = None
+        return RevisionInfo(branch, revno, revision_id)
+
 
 # classes in this list should have a "prefix" attribute, against which
 # string specs are matched
@@ -253,7 +266,8 @@ class RevisionSpec_revno(RevisionSpec):
     A negative number will count from the end of the branch (-1 is the
     last revision, -2 the previous one). If the negative number is larger
     than the branch's history, the first revision is returned.
-    examples:
+    Examples::
+
       revno:1                   -> return the first revision
       revno:3:/path/to/branch   -> return the 3rd revision of
                                    the branch '/path/to/branch'
@@ -308,14 +322,10 @@ class RevisionSpec_revno(RevisionSpec):
         if dotted:
             branch.lock_read()
             try:
-                last_rev = branch.last_revision()
-                merge_sorted_revisions = tsort.merge_sort(
-                    branch.repository.get_revision_graph(last_rev),
-                    last_rev,
-                    generate_revno=True)
-                def match(item):
-                    return item[3] == match_revno
-                revisions = filter(match, merge_sorted_revisions)
+                revision_id_to_revno = branch.get_revision_id_to_revno_map()
+                revisions = [revision_id for revision_id, revno
+                             in revision_id_to_revno.iteritems()
+                             if revno == match_revno]
             finally:
                 branch.unlock()
             if len(revisions) != 1:
@@ -323,9 +333,11 @@ class RevisionSpec_revno(RevisionSpec):
             else:
                 # there is no traditional 'revno' for dotted-decimal revnos.
                 # so for  API compatability we return None.
-                return RevisionInfo(branch, None, revisions[0][1])
+                return RevisionInfo(branch, None, revisions[0])
         else:
             if revno < 0:
+                # if get_rev_id supported negative revnos, there would not be a
+                # need for this special case.
                 if (-revno) >= len(revs):
                     revno = 1
                 else:
@@ -359,17 +371,18 @@ class RevisionSpec_revid(RevisionSpec):
     Supply a specific revision id, that can be used to specify any
     revision id in the ancestry of the branch. 
     Including merges, and pending merges.
-    examples:
+    Examples::
+
       revid:aaaa@bbbb-123456789 -> Select revision 'aaaa@bbbb-123456789'
     """    
     prefix = 'revid:'
 
     def _match_on(self, branch, revs):
-        try:
-            revno = revs.index(self.spec) + 1
-        except ValueError:
-            revno = None
-        return RevisionInfo(branch, revno, self.spec)
+        # self.spec comes straight from parsing the command line arguments,
+        # so we expect it to be a Unicode string. Switch it to the internal
+        # representation.
+        revision_id = osutils.safe_revision_id(self.spec, warn=False)
+        return RevisionInfo.from_revision_id(branch, revision_id, revs)
 
 SPEC_TYPES.append(RevisionSpec_revid)
 
@@ -381,7 +394,8 @@ class RevisionSpec_last(RevisionSpec):
 
     Supply a positive number to get the nth revision from the end.
     This is the same as supplying negative numbers to the 'revno:' spec.
-    examples:
+    Examples::
+
       last:1        -> return the last revision
       last:3        -> return the revision 2 before the end.
     """    
@@ -422,7 +436,8 @@ class RevisionSpec_before(RevisionSpec):
     This is mostly useful when inspecting revisions that are not in the
     revision history of a branch.
 
-    examples:
+    Examples::
+
       before:1913    -> Return the parent of revno 1913 (revno 1912)
       before:revid:aaaa@bbbb-1234567890  -> return the parent of revision
                                             aaaa@bbbb-1234567890
@@ -443,7 +458,7 @@ class RevisionSpec_before(RevisionSpec):
             rev = branch.repository.get_revision(r.rev_id)
             if not rev.parent_ids:
                 revno = 0
-                revision_id = None
+                revision_id = revision.NULL_REVISION
             else:
                 revision_id = rev.parent_ids[0]
                 try:
@@ -463,16 +478,20 @@ SPEC_TYPES.append(RevisionSpec_before)
 
 
 class RevisionSpec_tag(RevisionSpec):
-    """To be implemented."""
+    """Select a revision identified by tag name"""
 
-    help_txt = """To be implemented."""
+    help_txt = """Selects a revision identified by a tag name.
+
+    Tags are stored in the branch and created by the 'tag' command.
+    """
 
     prefix = 'tag:'
 
     def _match_on(self, branch, revs):
-        raise errors.InvalidRevisionSpec(self.user_spec, branch,
-                                         'tag: namespace registered,'
-                                         ' but not implemented')
+        # Can raise tags not supported, NoSuchTag, etc
+        return RevisionInfo.from_revision_id(branch,
+            branch.tags.lookup_tag(self.spec),
+            revs)
 
 SPEC_TYPES.append(RevisionSpec_tag)
 
@@ -506,10 +525,12 @@ class RevisionSpec_date(RevisionSpec):
     Matches the first entry after a given date (either at midnight or
     at a specified time).
 
-    One way to display all the changes since yesterday would be:
+    One way to display all the changes since yesterday would be::
+
         bzr log -r date:yesterday..-1
 
-    examples:
+    Examples::
+
       date:yesterday            -> select the first revision since yesterday
       date:2006-08-14,17:10:14  -> select the first revision after
                                    August 14th, 2006 at 5:10pm.
@@ -597,32 +618,49 @@ class RevisionSpec_ancestor(RevisionSpec):
     that your branch introduces, while excluding the changes that you
     have not merged from the remote branch.
 
-    examples:
+    Examples::
+
       ancestor:/path/to/branch
       $ bzr diff -r ancestor:../../mainline/branch
     """
     prefix = 'ancestor:'
 
     def _match_on(self, branch, revs):
+        trace.mutter('matching ancestor: on: %s, %s', self.spec, branch)
+        return self._find_revision_info(branch, self.spec)
+
+    @staticmethod
+    def _find_revision_info(branch, other_location):
         from bzrlib.branch import Branch
 
-        trace.mutter('matching ancestor: on: %s, %s', self.spec, branch)
-        other_branch = Branch.open(self.spec)
+        other_branch = Branch.open(other_location)
         revision_a = branch.last_revision()
         revision_b = other_branch.last_revision()
         for r, b in ((revision_a, branch), (revision_b, other_branch)):
             if r in (None, revision.NULL_REVISION):
                 raise errors.NoCommits(b)
-        revision_source = revision.MultipleRevisionSources(
-                branch.repository, other_branch.repository)
-        rev_id = revision.common_ancestor(revision_a, revision_b,
-                                          revision_source)
+        branch.lock_read()
+        other_branch.lock_read()
         try:
-            revno = branch.revision_id_to_revno(rev_id)
-        except errors.NoSuchRevision:
-            revno = None
-        return RevisionInfo(branch, revno, rev_id)
-        
+            graph = branch.repository.get_graph(other_branch.repository)
+            revision_a = revision.ensure_null(revision_a)
+            revision_b = revision.ensure_null(revision_b)
+            if revision.NULL_REVISION in (revision_a, revision_b):
+                rev_id = revision.NULL_REVISION
+            else:
+                rev_id = graph.find_unique_lca(revision_a, revision_b)
+                if rev_id == revision.NULL_REVISION:
+                    raise errors.NoCommonAncestor(revision_a, revision_b)
+            try:
+                revno = branch.revision_id_to_revno(rev_id)
+            except errors.NoSuchRevision:
+                revno = None
+            return RevisionInfo(branch, revno, rev_id)
+        finally:
+            branch.unlock()
+            other_branch.unlock()
+
+
 SPEC_TYPES.append(RevisionSpec_ancestor)
 
 
@@ -633,7 +671,8 @@ class RevisionSpec_branch(RevisionSpec):
 
     Supply the path to a branch to select its last revision.
 
-    examples:
+    Examples::
+
       branch:/path/to/branch
     """
     prefix = 'branch:'
@@ -653,3 +692,40 @@ class RevisionSpec_branch(RevisionSpec):
         return RevisionInfo(branch, revno, revision_b)
         
 SPEC_TYPES.append(RevisionSpec_branch)
+
+
+class RevisionSpec_submit(RevisionSpec_ancestor):
+    """Selects a common ancestor with a submit branch."""
+
+    help_txt = """Selects a common ancestor with the submit branch.
+
+    Diffing against this shows all the changes that were made in this branch,
+    and is a good predictor of what merge will do.  The submit branch is
+    used by the bundle and merge directive comands.  If no submit branch
+    is specified, the parent branch is used instead.
+
+    The common ancestor is the last revision that existed in both
+    branches. Usually this is the branch point, but it could also be
+    a revision that was merged.
+
+    Examples::
+
+      $ bzr diff -r submit:
+    """
+
+    prefix = 'submit:'
+
+    def _match_on(self, branch, revs):
+        trace.mutter('matching ancestor: on: %s, %s', self.spec, branch)
+        submit_location = branch.get_submit_branch()
+        location_type = 'submit branch'
+        if submit_location is None:
+            submit_location = branch.get_parent()
+            location_type = 'parent branch'
+        if submit_location is None:
+            raise errors.NoSubmitBranch(branch)
+        trace.note('Using %s %s', location_type, submit_location)
+        return self._find_revision_info(branch, submit_location)
+
+
+SPEC_TYPES.append(RevisionSpec_submit)
