@@ -19,8 +19,11 @@ import os
 
 import bzrlib
 from bzrlib import (
+    bzrdir,
     errors,
     lockdir,
+    osutils,
+    tests,
     )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
@@ -28,7 +31,7 @@ from bzrlib.commit import Commit, NullCommitReporter
 from bzrlib.config import BranchConfig
 from bzrlib.errors import (PointlessCommit, BzrError, SigningFailed, 
                            LockContention)
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import SymlinkFeature, TestCaseWithTransport
 from bzrlib.workingtree import WorkingTree
 
 
@@ -68,6 +71,9 @@ class CapturingReporter(NullCommitReporter):
     def renamed(self, change, old_path, new_path):
         self.calls.append(('renamed', change, old_path, new_path))
 
+    def is_verbose(self):
+        return True
+
 
 class TestCommit(TestCaseWithTransport):
 
@@ -90,11 +96,16 @@ class TestCommit(TestCaseWithTransport):
         eq(rev.message, 'add hello')
 
         tree1 = b.repository.revision_tree(rh[0])
+        tree1.lock_read()
         text = tree1.get_file_text(file_id)
-        eq(text, 'hello world')
+        tree1.unlock()
+        self.assertEqual('hello world', text)
 
         tree2 = b.repository.revision_tree(rh[1])
-        eq(tree2.get_file_text(file_id), 'version 2')
+        tree2.lock_read()
+        text = tree2.get_file_text(file_id)
+        tree2.unlock()
+        self.assertEqual('version 2', text)
 
     def test_delete_commit(self):
         """Test a commit with a deleted file"""
@@ -163,11 +174,15 @@ class TestCommit(TestCaseWithTransport):
         eq(b.revno(), 3)
 
         tree2 = b.repository.revision_tree('test@rev-2')
+        tree2.lock_read()
+        self.addCleanup(tree2.unlock)
         self.assertTrue(tree2.has_filename('hello'))
         self.assertEquals(tree2.get_file_text('hello-id'), 'hello')
         self.assertEquals(tree2.get_file_text('buongia-id'), 'new text')
         
         tree3 = b.repository.revision_tree('test@rev-3')
+        tree3.lock_read()
+        self.addCleanup(tree3.unlock)
         self.assertFalse(tree3.has_filename('hello'))
         self.assertEquals(tree3.get_file_text('buongia-id'), 'new text')
 
@@ -184,6 +199,8 @@ class TestCommit(TestCaseWithTransport):
 
         eq = self.assertEquals
         tree1 = b.repository.revision_tree('test@rev-1')
+        tree1.lock_read()
+        self.addCleanup(tree1.unlock)
         eq(tree1.id2path('hello-id'), 'hello')
         eq(tree1.get_file_text('hello-id'), 'contents of hello\n')
         self.assertFalse(tree1.has_filename('fruity'))
@@ -192,6 +209,8 @@ class TestCommit(TestCaseWithTransport):
         eq(ie.revision, 'test@rev-1')
 
         tree2 = b.repository.revision_tree('test@rev-2')
+        tree2.lock_read()
+        self.addCleanup(tree2.unlock)
         eq(tree2.id2path('hello-id'), 'fruity')
         eq(tree2.get_file_text('hello-id'), 'contents of hello\n')
         self.check_inventory_shape(tree2.inventory, ['fruity'])
@@ -221,28 +240,40 @@ class TestCommit(TestCaseWithTransport):
         wt.move(['hello'], 'a')
         r2 = 'test@rev-2'
         wt.commit('two', rev_id=r2, allow_pointless=False)
-        self.check_inventory_shape(wt.read_working_inventory(),
-                                   ['a', 'a/hello', 'b'])
+        wt.lock_read()
+        try:
+            self.check_inventory_shape(wt.read_working_inventory(),
+                                       ['a/', 'a/hello', 'b/'])
+        finally:
+            wt.unlock()
 
         wt.move(['b'], 'a')
         r3 = 'test@rev-3'
         wt.commit('three', rev_id=r3, allow_pointless=False)
-        self.check_inventory_shape(wt.read_working_inventory(),
-                                   ['a', 'a/hello', 'a/b'])
-        self.check_inventory_shape(b.repository.get_revision_inventory(r3),
-                                   ['a', 'a/hello', 'a/b'])
+        wt.lock_read()
+        try:
+            self.check_inventory_shape(wt.read_working_inventory(),
+                                       ['a/', 'a/hello', 'a/b/'])
+            self.check_inventory_shape(b.repository.get_revision_inventory(r3),
+                                       ['a/', 'a/hello', 'a/b/'])
+        finally:
+            wt.unlock()
 
         wt.move(['a/hello'], 'a/b')
         r4 = 'test@rev-4'
         wt.commit('four', rev_id=r4, allow_pointless=False)
-        self.check_inventory_shape(wt.read_working_inventory(),
-                                   ['a', 'a/b/hello', 'a/b'])
+        wt.lock_read()
+        try:
+            self.check_inventory_shape(wt.read_working_inventory(),
+                                       ['a/', 'a/b/hello', 'a/b/'])
+        finally:
+            wt.unlock()
 
         inv = b.repository.get_revision_inventory(r4)
         eq(inv['hello-id'].revision, r4)
         eq(inv['a-id'].revision, r1)
         eq(inv['b-id'].revision, r3)
-        
+
     def test_removed_commit(self):
         """Commit with a removed file"""
         wt = self.make_branch_and_tree('.')
@@ -343,8 +374,10 @@ class TestCommit(TestCaseWithTransport):
                                                       allow_pointless=True,
                                                       rev_id='B',
                                                       working_tree=wt)
-            self.assertEqual(Testament.from_revision(branch.repository,
-                             'B').as_short_text(),
+            def sign(text):
+                return bzrlib.gpg.LoopbackGPGStrategy(None).sign(text)
+            self.assertEqual(sign(Testament.from_revision(branch.repository,
+                             'B').as_short_text()),
                              branch.repository.get_signature_text('B'))
         finally:
             bzrlib.gpg.GPGStrategy = oldstrategy
@@ -509,9 +542,9 @@ class TestCommit(TestCaseWithTransport):
             ('change', 'added', 'newdir'),
             ('change', 'added', 'newfile'),
             ('renamed', 'renamed', 'dirtorename', 'renameddir'),
+            ('renamed', 'renamed', 'filetorename', 'renamedfile'),
             ('renamed', 'renamed', 'dirtoreparent', 'renameddir/reparenteddir'),
             ('renamed', 'renamed', 'filetoreparent', 'renameddir/reparentedfile'),
-            ('renamed', 'renamed', 'filetorename', 'renamedfile'),
             ('deleted', 'dirtoremove'),
             ('deleted', 'filetoremove'),
             ],
@@ -525,9 +558,13 @@ class TestCommit(TestCaseWithTransport):
         tree.commit('added a, b')
         tree.remove(['a', 'b'])
         tree.commit('removed a', specific_files='a')
-        basis = tree.basis_tree().inventory
-        self.assertIs(None, basis.path2id('a'))
-        self.assertFalse(basis.path2id('b') is None)
+        basis = tree.basis_tree()
+        tree.lock_read()
+        try:
+            self.assertIs(None, basis.path2id('a'))
+            self.assertFalse(basis.path2id('b') is None)
+        finally:
+            tree.unlock()
 
     def test_commit_saves_1ms_timestamp(self):
         """Passing in a timestamp is saved with 1ms resolution"""
@@ -552,6 +589,58 @@ class TestCommit(TestCaseWithTransport):
         timestamp_1ms = round(timestamp, 3)
         self.assertEqual(timestamp_1ms, timestamp)
 
+    def assertBasisTreeKind(self, kind, tree, file_id):
+        basis = tree.basis_tree()
+        basis.lock_read()
+        try:
+            self.assertEqual(kind, basis.kind(file_id))
+        finally:
+            basis.unlock()
+
+    def test_commit_kind_changes(self):
+        self.requireFeature(SymlinkFeature)
+        tree = self.make_branch_and_tree('.')
+        os.symlink('target', 'name')
+        tree.add('name', 'a-file-id')
+        tree.commit('Added a symlink')
+        self.assertBasisTreeKind('symlink', tree, 'a-file-id')
+
+        os.unlink('name')
+        self.build_tree(['name'])
+        tree.commit('Changed symlink to file')
+        self.assertBasisTreeKind('file', tree, 'a-file-id')
+
+        os.unlink('name')
+        os.symlink('target', 'name')
+        tree.commit('file to symlink')
+        self.assertBasisTreeKind('symlink', tree, 'a-file-id')
+
+        os.unlink('name')
+        os.mkdir('name')
+        tree.commit('symlink to directory')
+        self.assertBasisTreeKind('directory', tree, 'a-file-id')
+
+        os.rmdir('name')
+        os.symlink('target', 'name')
+        tree.commit('directory to symlink')
+        self.assertBasisTreeKind('symlink', tree, 'a-file-id')
+
+        # prepare for directory <-> file tests
+        os.unlink('name')
+        os.mkdir('name')
+        tree.commit('symlink to directory')
+        self.assertBasisTreeKind('directory', tree, 'a-file-id')
+
+        os.rmdir('name')
+        self.build_tree(['name'])
+        tree.commit('Changed directory to file')
+        self.assertBasisTreeKind('file', tree, 'a-file-id')
+
+        os.unlink('name')
+        os.mkdir('name')
+        tree.commit('file to directory')
+        self.assertBasisTreeKind('directory', tree, 'a-file-id')
+
     def test_commit_unversioned_specified(self):
         """Commit should raise if specified files isn't in basis or worktree"""
         tree = self.make_branch_and_tree('.')
@@ -608,3 +697,55 @@ class TestCommit(TestCaseWithTransport):
         repository.add_inventory = raise_
         self.assertRaises(errors.NoSuchFile, tree.commit, message_callback=cb)
         self.assertFalse(cb.called)
+
+    def test_selected_file_merge_commit(self):
+        """Ensure the correct error is raised"""
+        tree = self.make_branch_and_tree('foo')
+        # pending merge would turn into a left parent
+        tree.commit('commit 1')
+        tree.add_parent_tree_id('example')
+        self.build_tree(['foo/bar', 'foo/baz'])
+        tree.add(['bar', 'baz'])
+        err = self.assertRaises(errors.CannotCommitSelectedFileMerge,
+            tree.commit, 'commit 2', specific_files=['bar', 'baz'])
+        self.assertEqual(['bar', 'baz'], err.files)
+        self.assertEqual('Selected-file commit of merges is not supported'
+                         ' yet: files bar, baz', str(err))
+
+    def test_commit_ordering(self):
+        """Test of corner-case commit ordering error"""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/z/', 'a/c/', 'a/z/x', 'a/z/y'])
+        tree.add(['a/', 'a/z/', 'a/c/', 'a/z/x', 'a/z/y'])
+        tree.commit('setup')
+        self.build_tree(['a/c/d/'])
+        tree.add('a/c/d')
+        tree.rename_one('a/z/x', 'a/c/d/x')
+        tree.commit('test', specific_files=['a/z/y'])
+ 
+    def test_commit_no_author(self):
+        """The default kwarg author in MutableTree.commit should not add
+        the 'author' revision property.
+        """
+        tree = self.make_branch_and_tree('foo')
+        rev_id = tree.commit('commit 1')
+        rev = tree.branch.repository.get_revision(rev_id)
+        self.assertFalse('author' in rev.properties)
+
+    def test_commit_author(self):
+        """Passing a non-empty author kwarg to MutableTree.commit should add
+        the 'author' revision property.
+        """
+        tree = self.make_branch_and_tree('foo')
+        rev_id = tree.commit('commit 1', author='John Doe <jdoe@example.com>')
+        rev = tree.branch.repository.get_revision(rev_id)
+        self.assertEqual('John Doe <jdoe@example.com>',
+                         rev.properties['author'])
+
+    def test_commit_with_checkout_and_branch_sharing_repo(self):
+        repo = self.make_repository('repo', shared=True)
+        # make_branch_and_tree ignores shared repos
+        branch = bzrdir.BzrDir.create_branch_convenience('repo/branch')
+        tree2 = branch.create_checkout('repo/tree2')
+        tree2.commit('message', rev_id='rev1')
+        self.assertTrue(tree2.branch.repository.has_revision('rev1'))
