@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006 Canonical Ltd
+# Copyright (C) 2004, 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,11 +19,17 @@ import os
 import time
 
 from bzrlib import (
+    branch,
+    bzrdir,
     errors,
+    repository,
     )
-from bzrlib.builtins import merge
 from bzrlib.tests import TestCase, TestCaseWithTransport
-from bzrlib.revisionspec import RevisionSpec, RevisionSpec_revno
+from bzrlib.revisionspec import (
+    RevisionSpec,
+    RevisionSpec_revno,
+    RevisionSpec_tag,
+    )
 
 
 def spec_in_history(spec, branch):
@@ -60,11 +66,11 @@ class TestRevisionSpec(TestCaseWithTransport):
     def assertInHistoryIs(self, exp_revno, exp_revision_id, revision_spec):
         rev_info = self.get_in_history(revision_spec)
         self.assertEqual(exp_revno, rev_info.revno,
-                         'Revision spec: %s returned wrong revno: %s != %s'
+                         'Revision spec: %r returned wrong revno: %r != %r'
                          % (revision_spec, exp_revno, rev_info.revno))
         self.assertEqual(exp_revision_id, rev_info.rev_id,
-                         'Revision spec: %s returned wrong revision id:'
-                         ' %s != %s'
+                         'Revision spec: %r returned wrong revision id:'
+                         ' %r != %r'
                          % (revision_spec, exp_revision_id, rev_info.rev_id))
 
     def assertInvalid(self, revision_spec, extra=''):
@@ -110,7 +116,7 @@ class TestRevnoFromString(TestCase):
 class TestRevisionSpec_revno(TestRevisionSpec):
 
     def test_positive_int(self):
-        self.assertInHistoryIs(0, None, '0')
+        self.assertInHistoryIs(0, 'null:', '0')
         self.assertInHistoryIs(1, 'r1', '1')
         self.assertInHistoryIs(2, 'r2', '2')
         self.assertInvalid('3')
@@ -127,7 +133,7 @@ class TestRevisionSpec_revno(TestRevisionSpec):
         self.assertInHistoryIs(1, 'r1', '-100')
 
     def test_positive(self):
-        self.assertInHistoryIs(0, None, 'revno:0')
+        self.assertInHistoryIs(0, 'null:', 'revno:0')
         self.assertInHistoryIs(1, 'r1', 'revno:1')
         self.assertInHistoryIs(2, 'r2', 'revno:2')
 
@@ -270,13 +276,20 @@ class TestRevisionSpec_revid(TestRevisionSpec):
                                           revision_id='alt_r3')
         self.assertInHistoryIs(None, 'alt_r3', 'revid:alt_r3')
 
+    def test_unicode(self):
+        """We correctly convert a unicode ui string to an encoded revid."""
+        revision_id = u'\N{SNOWMAN}'.encode('utf-8')
+        self.tree.commit('unicode', rev_id=revision_id)
+        self.assertInHistoryIs(3, revision_id, u'revid:\N{SNOWMAN}')
+        self.assertInHistoryIs(3, revision_id, 'revid:' + revision_id)
+
 
 class TestRevisionSpec_last(TestRevisionSpec):
 
     def test_positive(self):
         self.assertInHistoryIs(2, 'r2', 'last:1')
         self.assertInHistoryIs(1, 'r1', 'last:2')
-        self.assertInHistoryIs(0, None, 'last:3')
+        self.assertInHistoryIs(0, 'null:', 'last:3')
 
     def test_empty(self):
         self.assertInHistoryIs(2, 'r2', 'last:')
@@ -309,7 +322,7 @@ class TestRevisionSpec_before(TestRevisionSpec):
         self.assertInHistoryIs(1, 'r1', 'before:-1')
 
     def test_before_one(self):
-        self.assertInHistoryIs(0, None, 'before:1')
+        self.assertInHistoryIs(0, 'null:', 'before:1')
 
     def test_before_none(self):
         self.assertInvalid('before:0',
@@ -330,14 +343,31 @@ class TestRevisionSpec_before(TestRevisionSpec):
         new_tree.commit('first', rev_id='new_r1')
         self.tree.branch.repository.fetch(new_tree.branch.repository,
                                           revision_id='new_r1')
-        self.assertInHistoryIs(0, None, 'before:revid:new_r1')
+        self.assertInHistoryIs(0, 'null:', 'before:revid:new_r1')
 
 
 class TestRevisionSpec_tag(TestRevisionSpec):
     
-    def test_invalid(self):
-        self.assertInvalid('tag:foo', extra='\ntag: namespace registered,'
-                                            ' but not implemented')
+    def make_branch_and_tree(self, relpath):
+        # override format as the default one may not support tags
+        return TestRevisionSpec.make_branch_and_tree(
+            self, relpath, format='dirstate-tags')
+
+    def test_from_string_tag(self):
+        spec = RevisionSpec.from_string('tag:bzr-0.14')
+        self.assertIsInstance(spec, RevisionSpec_tag)
+        self.assertEqual(spec.spec, 'bzr-0.14')
+
+    def test_lookup_tag(self):
+        self.tree.branch.tags.set_tag('bzr-0.14', 'r1')
+        self.assertInHistoryIs(1, 'r1', 'tag:bzr-0.14')
+
+    def test_failed_lookup(self):
+        # tags that don't exist give a specific message: arguably we should
+        # just give InvalidRevisionSpec but I think this is more helpful
+        self.assertRaises(errors.NoSuchTag,
+            self.get_in_history,
+            'tag:some-random-tag')
 
 
 class TestRevisionSpec_date(TestRevisionSpec):
@@ -451,3 +481,19 @@ class TestRevisionSpec_branch(TestRevisionSpec):
         new_tree = self.make_branch_and_tree('new_tree')
         self.assertRaises(errors.NoCommits,
                           self.get_in_history, 'branch:new_tree')
+
+
+class TestRevisionSpec_submit(TestRevisionSpec):
+
+    def test_submit_branch(self):
+        # Common ancestor of trees is 'alt_r2'
+        self.assertRaises(errors.NoSubmitBranch, self.get_in_history,
+                          'submit:')
+        self.tree.branch.set_parent('../tree2')
+        self.assertInHistoryIs(None, 'alt_r2', 'submit:')
+        self.tree.branch.set_parent('bogus')
+        self.assertRaises(errors.NotBranchError, self.get_in_history,
+            'submit:')
+        # submit branch overrides parent branch
+        self.tree.branch.set_submit_branch('tree2')
+        self.assertInHistoryIs(None, 'alt_r2', 'submit:')
