@@ -21,6 +21,7 @@
 from bzrlib.tests import TestCase
 from bzrlib.tsort import topo_sort, TopoSorter, MergeSorter, merge_sort
 from bzrlib.errors import GraphCycleError
+from bzrlib.revision import NULL_REVISION
 
 
 class TopoSortTests(TestCase):
@@ -85,6 +86,12 @@ class TopoSortTests(TestCase):
                                    (8, [0, 1, 4, 5, 6])]),
                                   [0, 1, 2, 3, 4, 5, 6, 7, 8])
 
+    def test_tsort_unincluded_parent(self):
+        """Sort nodes, but don't include some parents in the output"""
+        self.assertSortAndIterate([(0, [1]),
+                                   (1, [2])],
+                                   [1, 0])
+
 
 class MergeSortTests(TestCase):
 
@@ -106,6 +113,8 @@ class MergeSortTests(TestCase):
         # sorting of an emptygraph does not error
         self.assertSortAndIterate({}, None, [], False)
         self.assertSortAndIterate({}, None, [], True)
+        self.assertSortAndIterate({}, NULL_REVISION, [], False)
+        self.assertSortAndIterate({}, NULL_REVISION, [], True)
 
     def test_merge_sort_not_empty_no_tip(self):
         # merge sorting of a branch starting with None should result
@@ -175,7 +184,55 @@ class MergeSortTests(TestCase):
              ],
             True
             )
-        
+
+    def test_merge_sort_race(self):
+        # A
+        # |
+        # B-.
+        # |\ \
+        # | | C
+        # | |/
+        # | D
+        # |/
+        # F
+        graph = {'A': [],
+                 'B': ['A'],
+                 'C': ['B'],
+                 'D': ['B', 'C'],
+                 'F': ['B', 'D'],
+                 }
+        self.assertSortAndIterate(graph, 'F',
+            [(0, 'F', 0, (3,), False),
+             (1, 'D', 1, (2,2,1), False),
+             (2, 'C', 1, (2,1,1), True), # XXX: Shouldn't it be merge_depth=2?
+             (3, 'B', 0, (2,), False),
+             (4, 'A', 0, (1,), True),
+             ], True)
+        # A
+        # |
+        # B-.
+        # |\ \
+        # | X C
+        # | |/
+        # | D
+        # |/
+        # F
+        graph = {'A': [],
+                 'B': ['A'],
+                 'C': ['B'],
+                 'X': ['B'],
+                 'D': ['X', 'C'],
+                 'F': ['B', 'D'],
+                 }
+        self.assertSortAndIterate(graph, 'F',
+            [(0, 'F', 0, (3,), False),
+             (1, 'D', 1, (2,1,2), False),
+             (2, 'C', 2, (2,2,1), True),
+             (3, 'X', 1, (2,1,1), True),
+             (4, 'B', 0, (2,), False),
+             (5, 'A', 0, (1,), True),
+             ], True)
+
     def test_merge_depth_with_nested_merges(self):
         # the merge depth marker should reflect the depth of the revision
         # in terms of merges out from the mainline
@@ -222,17 +279,96 @@ class MergeSortTests(TestCase):
              }.items(),
             'A',
             [(0, 'A', 0, (3,),  False),
-             (1, 'B', 1, (1,2,2), False),
-             (2, 'C', 1, (1,2,1), True),
+             (1, 'B', 1, (1,3,2), False),
+             (2, 'C', 1, (1,3,1), True),
              (3, 'D', 0, (2,), False),
              (4, 'E', 1, (1,1,2), False),
-             (5, 'F', 2, (1,1,1,1,1), True),
+             (5, 'F', 2, (1,2,1), True),
              (6, 'G', 1, (1,1,1), True),
              (7, 'H', 0, (1,), True),
              ],
             True
             )
- 
+
+    def test_dotted_revnos_with_simple_merges(self):
+        # A         1
+        # |\
+        # B C       2, 1.1.1
+        # | |\
+        # D E F     3, 1.1.2, 1.2.1
+        # |/ /|
+        # G H I     4, 1.2.2, 1.3.1
+        # |/ /
+        # J K       5, 1.3.2
+        # |/
+        # L         6
+        self.assertSortAndIterate(
+            {'A': [],
+             'B': ['A'],
+             'C': ['A'],
+             'D': ['B'],
+             'E': ['C'],
+             'F': ['C'],
+             'G': ['D', 'E'],
+             'H': ['F'],
+             'I': ['F'],
+             'J': ['G', 'H'],
+             'K': ['I'],
+             'L': ['J', 'K'],
+            }.items(),
+            'L',
+            [(0, 'L', 0, (6,), False),
+             (1, 'K', 1, (1,3,2), False),
+             (2, 'I', 1, (1,3,1), True),
+             (3, 'J', 0, (5,), False),
+             (4, 'H', 1, (1,2,2), False),
+             (5, 'F', 1, (1,2,1), True),
+             (6, 'G', 0, (4,), False),
+             (7, 'E', 1, (1,1,2), False),
+             (8, 'C', 1, (1,1,1), True),
+             (9, 'D', 0, (3,), False),
+             (10, 'B', 0, (2,), False),
+             (11, 'A', 0, (1,),  True),
+             ],
+            True
+            )
+        # Adding a shortcut from the first revision should not change any of
+        # the existing numbers
+        self.assertSortAndIterate(
+            {'A': [],
+             'B': ['A'],
+             'C': ['A'],
+             'D': ['B'],
+             'E': ['C'],
+             'F': ['C'],
+             'G': ['D', 'E'],
+             'H': ['F'],
+             'I': ['F'],
+             'J': ['G', 'H'],
+             'K': ['I'],
+             'L': ['J', 'K'],
+             'M': ['A'],
+             'N': ['L', 'M'],
+            }.items(),
+            'N',
+            [(0, 'N', 0, (7,), False),
+             (1, 'M', 1, (1,4,1), True),
+             (2, 'L', 0, (6,), False),
+             (3, 'K', 1, (1,3,2), False),
+             (4, 'I', 1, (1,3,1), True),
+             (5, 'J', 0, (5,), False),
+             (6, 'H', 1, (1,2,2), False),
+             (7, 'F', 1, (1,2,1), True),
+             (8, 'G', 0, (4,), False),
+             (9, 'E', 1, (1,1,2), False),
+             (10, 'C', 1, (1,1,1), True),
+             (11, 'D', 0, (3,), False),
+             (12, 'B', 0, (2,), False),
+             (13, 'A', 0, (1,),  True),
+             ],
+            True
+            )
+
     def test_end_of_merge_not_last_revision_in_branch(self):
         # within a branch only the last revision gets an
         # end of merge marker.
@@ -305,11 +441,11 @@ class MergeSortTests(TestCase):
              },
             'A',
             [(0, 'A', 0, (2,), False),
-             (1, 'B', 1, (1,2,2), False),
-             (2, 'C', 2, (1,2,1,1,1), True),
-             (3, 'D', 1, (1,2,1), True),
+             (1, 'B', 1, (1,3,2), False),
+             (2, 'C', 2, (1,4,1), True),
+             (3, 'D', 1, (1,3,1), True),
              (4, 'E', 1, (1,1,2), False),
-             (5, 'F', 2, (1,1,1,1,1), True),
+             (5, 'F', 2, (1,2,1), True),
              (6, 'G', 1, (1,1,1), True),
              (7, 'H', 0, (1,), True),
              ],
