@@ -1,5 +1,5 @@
 # Copyright (C) 2005 Robey Pointer <robey@lag.net>
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,17 +24,13 @@ import socket
 import subprocess
 import sys
 
-from bzrlib.config import config_dir, ensure_config_dir_exists
-from bzrlib.errors import (ConnectionError,
-                           ParamikoNotPresent,
-                           SocketConnectionError,
-                           TransportError,
-                           UnknownSSH,
-                           )
-
-from bzrlib.osutils import pathjoin
-from bzrlib.trace import mutter, warning
-import bzrlib.ui
+from bzrlib import (
+    config,
+    errors,
+    osutils,
+    trace,
+    ui,
+    )
 
 try:
     import paramiko
@@ -58,55 +54,114 @@ _paramiko_version = getattr(paramiko, '__version_info__', (0, 0, 0))
 # connect to an agent if we are on win32 and using Paramiko older than 1.6
 _use_ssh_agent = (sys.platform != 'win32' or _paramiko_version >= (1, 6, 0))
 
-_ssh_vendors = {}
 
-def register_ssh_vendor(name, vendor):
-    """Register SSH vendor."""
-    _ssh_vendors[name] = vendor
+class SSHVendorManager(object):
+    """Manager for manage SSH vendors."""
 
-    
-_ssh_vendor = None
-def _get_ssh_vendor():
-    """Find out what version of SSH is on the system."""
-    global _ssh_vendor
-    if _ssh_vendor is not None:
-        return _ssh_vendor
+    # Note, although at first sign the class interface seems similar to
+    # bzrlib.registry.Registry it is not possible/convenient to directly use
+    # the Registry because the class just has "get()" interface instead of the
+    # Registry's "get(key)".
 
-    if 'BZR_SSH' in os.environ:
-        vendor_name = os.environ['BZR_SSH']
+    def __init__(self):
+        self._ssh_vendors = {}
+        self._cached_ssh_vendor = None
+        self._default_ssh_vendor = None
+
+    def register_default_vendor(self, vendor):
+        """Register default SSH vendor."""
+        self._default_ssh_vendor = vendor
+
+    def register_vendor(self, name, vendor):
+        """Register new SSH vendor by name."""
+        self._ssh_vendors[name] = vendor
+
+    def clear_cache(self):
+        """Clear previously cached lookup result."""
+        self._cached_ssh_vendor = None
+
+    def _get_vendor_by_environment(self, environment=None):
+        """Return the vendor or None based on BZR_SSH environment variable.
+
+        :raises UnknownSSH: if the BZR_SSH environment variable contains
+                            unknown vendor name
+        """
+        if environment is None:
+            environment = os.environ
+        if 'BZR_SSH' in environment:
+            vendor_name = environment['BZR_SSH']
+            try:
+                vendor = self._ssh_vendors[vendor_name]
+            except KeyError:
+                raise errors.UnknownSSH(vendor_name)
+            return vendor
+        return None
+
+    def _get_ssh_version_string(self, args):
+        """Return SSH version string from the subprocess."""
         try:
-            _ssh_vendor = _ssh_vendors[vendor_name]
-        except KeyError:
-            raise UnknownSSH(vendor_name)
-        return _ssh_vendor
+            p = subprocess.Popen(args,
+                                 stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE,
+                                 **os_specific_subprocess_params())
+            stdout, stderr = p.communicate()
+        except OSError:
+            stdout = stderr = ''
+        return stdout + stderr
 
-    try:
-        p = subprocess.Popen(['ssh', '-V'],
-                             stdin=subprocess.PIPE,
-                             stdout=subprocess.PIPE,
-                             stderr=subprocess.PIPE,
-                             **os_specific_subprocess_params())
-        returncode = p.returncode
-        stdout, stderr = p.communicate()
-    except OSError:
-        returncode = -1
-        stdout = stderr = ''
-    if 'OpenSSH' in stderr:
-        mutter('ssh implementation is OpenSSH')
-        _ssh_vendor = OpenSSHSubprocessVendor()
-    elif 'SSH Secure Shell' in stderr:
-        mutter('ssh implementation is SSH Corp.')
-        _ssh_vendor = SSHCorpSubprocessVendor()
+    def _get_vendor_by_version_string(self, version, args):
+        """Return the vendor or None based on output from the subprocess.
 
-    if _ssh_vendor is not None:
-        return _ssh_vendor
+        :param version: The output of 'ssh -V' like command.
+        :param args: Command line that was run.
+        """
+        vendor = None
+        if 'OpenSSH' in version:
+            trace.mutter('ssh implementation is OpenSSH')
+            vendor = OpenSSHSubprocessVendor()
+        elif 'SSH Secure Shell' in version:
+            trace.mutter('ssh implementation is SSH Corp.')
+            vendor = SSHCorpSubprocessVendor()
+        elif 'plink' in version and args[0] == 'plink':
+            # Checking if "plink" was the executed argument as Windows
+            # sometimes reports 'ssh -V' incorrectly with 'plink' in it's
+            # version.  See https://bugs.launchpad.net/bzr/+bug/107155
+            trace.mutter("ssh implementation is Putty's plink.")
+            vendor = PLinkSubprocessVendor()
+        return vendor
 
-    # XXX: 20051123 jamesh
-    # A check for putty's plink or lsh would go here.
+    def _get_vendor_by_inspection(self):
+        """Return the vendor or None by checking for known SSH implementations."""
+        for args in (['ssh', '-V'], ['plink', '-V']):
+            version = self._get_ssh_version_string(args)
+            vendor = self._get_vendor_by_version_string(version, args)
+            if vendor is not None:
+                return vendor
+        return None
 
-    mutter('falling back to paramiko implementation')
-    _ssh_vendor = ParamikoVendor()
-    return _ssh_vendor
+    def get_vendor(self, environment=None):
+        """Find out what version of SSH is on the system.
+
+        :raises SSHVendorNotFound: if no any SSH vendor is found
+        :raises UnknownSSH: if the BZR_SSH environment variable contains
+                            unknown vendor name
+        """
+        if self._cached_ssh_vendor is None:
+            vendor = self._get_vendor_by_environment(environment)
+            if vendor is None:
+                vendor = self._get_vendor_by_inspection()
+                if vendor is None:
+                    trace.mutter('falling back to default implementation')
+                    vendor = self._default_ssh_vendor
+                    if vendor is None:
+                        raise errors.SSHVendorNotFound()
+            self._cached_ssh_vendor = vendor
+        return self._cached_ssh_vendor
+
+_ssh_vendor_manager = SSHVendorManager()
+_get_ssh_vendor = _ssh_vendor_manager.get_vendor
+register_default_ssh_vendor = _ssh_vendor_manager.register_default_vendor
+register_ssh_vendor = _ssh_vendor_manager.register_vendor
 
 
 def _ignore_sigint():
@@ -115,7 +170,6 @@ def _ignore_sigint():
     # <https://launchpad.net/products/bzr/+bug/41433/+index>
     import signal
     signal.signal(signal.SIGINT, signal.SIG_IGN)
-    
 
 
 class LoopbackSFTP(object):
@@ -123,7 +177,7 @@ class LoopbackSFTP(object):
 
     def __init__(self, sock):
         self.__socket = sock
- 
+
     def send(self, data):
         return self.__socket.send(data)
 
@@ -139,7 +193,7 @@ class LoopbackSFTP(object):
 
 class SSHVendor(object):
     """Abstract base class for SSH vendor implementations."""
-    
+
     def connect_sftp(self, username, password, host, port):
         """Make an SSH connection, and return an SFTPClient.
         
@@ -162,7 +216,7 @@ class SSHVendor(object):
             method that returns a pair of (read, write) filelike objects.
         """
         raise NotImplementedError(self.connect_ssh)
-        
+
     def _raise_connection_error(self, host, port=None, orig_error=None,
                                 msg='Unable to connect to SSH host'):
         """Raise a SocketConnectionError with properly formatted host.
@@ -170,13 +224,13 @@ class SSHVendor(object):
         This just unifies all the locations that try to raise ConnectionError,
         so that they format things properly.
         """
-        raise SocketConnectionError(host=host, port=port, msg=msg,
-                                    orig_error=orig_error)
+        raise errors.SocketConnectionError(host=host, port=port, msg=msg,
+                                           orig_error=orig_error)
 
 
 class LoopbackVendor(SSHVendor):
     """SSH "vendor" that connects over a plain TCP socket, not SSH."""
-    
+
     def connect_sftp(self, username, password, host, port):
         sock = socket.socket()
         try:
@@ -204,7 +258,7 @@ class ParamikoVendor(SSHVendor):
 
     def _connect(self, username, password, host, port):
         global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
-        
+
         load_host_keys()
 
         try:
@@ -213,36 +267,41 @@ class ParamikoVendor(SSHVendor):
             t.start_client()
         except (paramiko.SSHException, socket.error), e:
             self._raise_connection_error(host, port=port, orig_error=e)
-            
+
         server_key = t.get_remote_server_key()
         server_key_hex = paramiko.util.hexify(server_key.get_fingerprint())
         keytype = server_key.get_name()
         if host in SYSTEM_HOSTKEYS and keytype in SYSTEM_HOSTKEYS[host]:
             our_server_key = SYSTEM_HOSTKEYS[host][keytype]
-            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+            our_server_key_hex = paramiko.util.hexify(
+                our_server_key.get_fingerprint())
         elif host in BZR_HOSTKEYS and keytype in BZR_HOSTKEYS[host]:
             our_server_key = BZR_HOSTKEYS[host][keytype]
-            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+            our_server_key_hex = paramiko.util.hexify(
+                our_server_key.get_fingerprint())
         else:
-            warning('Adding %s host key for %s: %s' % (keytype, host, server_key_hex))
+            trace.warning('Adding %s host key for %s: %s'
+                          % (keytype, host, server_key_hex))
             add = getattr(BZR_HOSTKEYS, 'add', None)
             if add is not None: # paramiko >= 1.X.X
                 BZR_HOSTKEYS.add(host, keytype, server_key)
             else:
                 BZR_HOSTKEYS.setdefault(host, {})[keytype] = server_key
             our_server_key = server_key
-            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+            our_server_key_hex = paramiko.util.hexify(
+                our_server_key.get_fingerprint())
             save_host_keys()
         if server_key != our_server_key:
             filename1 = os.path.expanduser('~/.ssh/known_hosts')
-            filename2 = pathjoin(config_dir(), 'ssh_host_keys')
-            raise TransportError('Host keys for %s do not match!  %s != %s' % \
+            filename2 = osutils.pathjoin(config.config_dir(), 'ssh_host_keys')
+            raise errors.TransportError(
+                'Host keys for %s do not match!  %s != %s' %
                 (host, our_server_key_hex, server_key_hex),
                 ['Try editing %s or %s' % (filename1, filename2)])
 
-        _paramiko_auth(username, password, host, t)
+        _paramiko_auth(username, password, host, port, t)
         return t
-        
+
     def connect_sftp(self, username, password, host, port):
         t = self._connect(username, password, host, port)
         try:
@@ -263,12 +322,19 @@ class ParamikoVendor(SSHVendor):
                                          msg='Unable to invoke remote bzr')
 
 if paramiko is not None:
-    register_ssh_vendor('paramiko', ParamikoVendor())
+    vendor = ParamikoVendor()
+    register_ssh_vendor('paramiko', vendor)
+    register_ssh_vendor('none', vendor)
+    register_default_ssh_vendor(vendor)
+    _sftp_connection_errors = (EOFError, paramiko.SSHException)
+    del vendor
+else:
+    _sftp_connection_errors = (EOFError,)
 
 
 class SubprocessVendor(SSHVendor):
     """Abstract base class for vendors that use pipes to a subprocess."""
-    
+
     def _connect(self, argv):
         proc = subprocess.Popen(argv,
                                 stdin=subprocess.PIPE,
@@ -282,7 +348,7 @@ class SubprocessVendor(SSHVendor):
                                                   subsystem='sftp')
             sock = self._connect(argv)
             return SFTPClient(sock)
-        except (EOFError, paramiko.SSHException), e:
+        except _sftp_connection_errors, e:
             self._raise_connection_error(host, port=port, orig_error=e)
         except (OSError, IOError), e:
             # If the machine is fast enough, ssh can actually exit
@@ -315,12 +381,10 @@ class SubprocessVendor(SSHVendor):
         """
         raise NotImplementedError(self._get_vendor_specific_argv)
 
-register_ssh_vendor('none', ParamikoVendor())
-
 
 class OpenSSHSubprocessVendor(SubprocessVendor):
     """SSH vendor that uses the 'ssh' executable from OpenSSH."""
-    
+
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
         assert subsystem is not None or command is not None, (
@@ -365,29 +429,59 @@ class SSHCorpSubprocessVendor(SubprocessVendor):
         else:
             args.extend([host] + command)
         return args
-    
+
 register_ssh_vendor('ssh', SSHCorpSubprocessVendor())
 
 
-def _paramiko_auth(username, password, host, paramiko_transport):
+class PLinkSubprocessVendor(SubprocessVendor):
+    """SSH vendor that uses the 'plink' executable from Putty."""
+
+    def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
+                                  command=None):
+        assert subsystem is not None or command is not None, (
+            'Must specify a command or subsystem')
+        if subsystem is not None:
+            assert command is None, (
+                'subsystem and command are mutually exclusive')
+        args = ['plink', '-x', '-a', '-ssh', '-2', '-batch']
+        if port is not None:
+            args.extend(['-P', str(port)])
+        if username is not None:
+            args.extend(['-l', username])
+        if subsystem is not None:
+            args.extend(['-s', host, subsystem])
+        else:
+            args.extend([host] + command)
+        return args
+
+register_ssh_vendor('plink', PLinkSubprocessVendor())
+
+
+def _paramiko_auth(username, password, host, port, paramiko_transport):
     # paramiko requires a username, but it might be none if nothing was supplied
     # use the local username, just in case.
     # We don't override username, because if we aren't using paramiko,
     # the username might be specified in ~/.ssh/config and we don't want to
     # force it to something else
     # Also, it would mess up the self.relpath() functionality
-    username = username or getpass.getuser()
+    auth = config.AuthenticationConfig()
+    if username is None:
+        username = auth.get_user('ssh', host, port=port)
+        if username is None:
+            # Default to local user
+            username = getpass.getuser()
 
     if _use_ssh_agent:
         agent = paramiko.Agent()
         for key in agent.get_keys():
-            mutter('Trying SSH agent key %s' % paramiko.util.hexify(key.get_fingerprint()))
+            trace.mutter('Trying SSH agent key %s'
+                         % paramiko.util.hexify(key.get_fingerprint()))
             try:
                 paramiko_transport.auth_publickey(username, key)
                 return
             except paramiko.SSHException, e:
                 pass
-    
+
     # okay, try finding id_rsa or id_dss?  (posix only)
     if _try_pkey_auth(paramiko_transport, paramiko.RSAKey, username, 'id_rsa'):
         return
@@ -402,14 +496,12 @@ def _paramiko_auth(username, password, host, paramiko_transport):
             pass
 
     # give up and ask for a password
-    password = bzrlib.ui.ui_factory.get_password(
-            prompt='SSH %(user)s@%(host)s password',
-            user=username, host=host)
+    password = auth.get_password('ssh', host, username, port=port)
     try:
         paramiko_transport.auth_password(username, password)
     except paramiko.SSHException, e:
-        raise ConnectionError('Unable to authenticate to SSH host as %s@%s' %
-                              (username, host), e)
+        raise errors.ConnectionError(
+            'Unable to authenticate to SSH host as %s@%s' % (username, host), e)
 
 
 def _try_pkey_auth(paramiko_transport, pkey_class, username, filename):
@@ -419,17 +511,18 @@ def _try_pkey_auth(paramiko_transport, pkey_class, username, filename):
         paramiko_transport.auth_publickey(username, key)
         return True
     except paramiko.PasswordRequiredException:
-        password = bzrlib.ui.ui_factory.get_password(
-                prompt='SSH %(filename)s password',
-                filename=filename)
+        password = ui.ui_factory.get_password(
+            prompt='SSH %(filename)s password', filename=filename)
         try:
             key = pkey_class.from_private_key_file(filename, password)
             paramiko_transport.auth_publickey(username, key)
             return True
         except paramiko.SSHException:
-            mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+            trace.mutter('SSH authentication via %s key failed.'
+                         % (os.path.basename(filename),))
     except paramiko.SSHException:
-        mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+        trace.mutter('SSH authentication via %s key failed.'
+                     % (os.path.basename(filename),))
     except IOError:
         pass
     return False
@@ -442,14 +535,15 @@ def load_host_keys():
     """
     global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
     try:
-        SYSTEM_HOSTKEYS = paramiko.util.load_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
-    except Exception, e:
-        mutter('failed to load system host keys: ' + str(e))
-    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
+        SYSTEM_HOSTKEYS = paramiko.util.load_host_keys(
+            os.path.expanduser('~/.ssh/known_hosts'))
+    except IOError, e:
+        trace.mutter('failed to load system host keys: ' + str(e))
+    bzr_hostkey_path = osutils.pathjoin(config.config_dir(), 'ssh_host_keys')
     try:
         BZR_HOSTKEYS = paramiko.util.load_host_keys(bzr_hostkey_path)
-    except Exception, e:
-        mutter('failed to load bzr host keys: ' + str(e))
+    except IOError, e:
+        trace.mutter('failed to load bzr host keys: ' + str(e))
         save_host_keys()
 
 
@@ -458,8 +552,8 @@ def save_host_keys():
     Save "discovered" host keys in $(config)/ssh_host_keys/.
     """
     global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
-    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
-    ensure_config_dir_exists()
+    bzr_hostkey_path = osutils.pathjoin(config.config_dir(), 'ssh_host_keys')
+    config.ensure_config_dir_exists()
 
     try:
         f = open(bzr_hostkey_path, 'w')
@@ -469,7 +563,7 @@ def save_host_keys():
                 f.write('%s %s %s\n' % (hostname, keytype, key.get_base64()))
         f.close()
     except IOError, e:
-        mutter('failed to save bzr host keys: ' + str(e))
+        trace.mutter('failed to save bzr host keys: ' + str(e))
 
 
 def os_specific_subprocess_params():
